/*
Copyright © 2022 Volodymyr Patuta me@vpatuta.xyz

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/
package cmd

import (
	"os"
	"time"

	"github.com/jedib0t/go-pretty/v6/table"
	"github.com/jedib0t/go-pretty/v6/text"
	"github.com/spf13/cobra"
	"golang.org/x/sys/unix"
)

// lsCmd represents the ls command
var lsCmd = &cobra.Command{
	Use:   "ls",
	Short: "List todos",
	Long: `A longer description that spans multiple lines and likely contains examples
and usage of using your command. For example:

Cobra is a CLI library for Go that empowers applications.
This application is a tool to generate the needed files
to quickly create a Cobra application.`,
	Run: func(cmd *cobra.Command, args []string) {
		todos := getAllTodos()
		ws, err := unix.IoctlGetWinsize(int(os.Stdin.Fd()), unix.TIOCGWINSZ)
		errPanic(err, "failed to determine terminal width")
		cols := int(ws.Col)
		t := table.NewWriter()
		t.SetOutputMirror(os.Stdout)
		t.AppendHeader(table.Row{"#", "Description", "Due date", "Completed"})
		for _, todo := range todos {
			date := todo.DueDate.Format("02/01/2006 15:04")
			d := time.Time{}
			if todo.DueDate == d {
				date = ""
			}
			desc := todo.Desc
			if len(desc) > (cols - 20) {
				desc = insertNth(desc, cols-20)
			}
			comp := '✗'
			if todo.IsDone {
				comp = '✓'
			}
			t.AppendRow([]interface{}{todo.ID, desc, date, string(comp)})
		}
		t.SetStyle(table.StyleLight)
		t.SetAllowedRowLength(cols)
		t.SetColumnConfigs([]table.ColumnConfig{
			{Number: 4, Align: text.AlignCenter},
		})
		t.Render()
	},
}

// insertNth inserts new line on each nth position.
func insertNth(s string, n int) string {
	for i := n - 1; i < len(s); i += n {
		s = s[:i] + "\n" + s[i:]
	}

	return s
}

func init() {
	rootCmd.AddCommand(lsCmd)
}
